<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://get4tell.com
 * @since      1.0.0
 *
 * @package    Personalization_4tell
 * @subpackage Personalization_4tell/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Personalization_4tell
 * @subpackage Personalization_4tell/admin
 * @author     Chris Reeder <chris.reeder@4-tell.com>
 */
class Personalization_4tell_Admin
{

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct($plugin_name, $version)
	{

		$this->plugin_name = $plugin_name;
		$this->version = $version;
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles()
	{
		wp_enqueue_style('wp-color-picker');
		wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) . 'css/personalization-4tell-admin.css', array(), $this->version, 'all');
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts()
	{

		wp_enqueue_script('jquery');
		wp_enqueue_script($this->plugin_name, plugin_dir_url(__FILE__) . 'js/personalization-4tell-admin.js', array('wp-color-picker', 'jquery'), $this->version, false);
	}

	/** This is the entrance function to the admins page */
	public function display_plugin_setup_page()
	{
		include_once('partials/personalization-4tell-admin-display.php');
	}

	public function load_update_checker()
	{
		include_once('plugin-update-checker/plugin-update-checker.php');
	}

	/** This adds a link to our Settings page to the WordPress Dashboard "Settings" tab */
	public function add_admin_menu()
	{
		add_options_page($this->plugin_name, "Adaptive Web", 'manage_options', $this->plugin_name, array($this, 'display_plugin_setup_page'));
	}

	public function add_action_links($links)
	{
		$settings_link = array(
			'<a href="' . admin_url('options-general.php?page=' . "Adaptive-Web") . '&tab=general_options">' . __('Settings', $this->plugin_name) . '</a>',
		);
		return array_merge($settings_link, $links);
	}

	/** This validates and santizes the form input for "General" and "Tray" options */
	public function options_update()
	{
		register_setting($this->plugin_name, $this->plugin_name, array($this, 'validate'));
	}

	public function checkForUpdates()
	{
		$UpdateChecker = new PluginUpdateChecker_3_1(
			'https://4tcdn.blob.core.windows.net/sites/actonact/wordpress-plugin/metadata.json',
			plugin_dir_path(dirname(__FILE__)) . "personalization-4tell.php", // This has to be to the top of the plugin
			'6'
		);
	}

	/** Remove and encode all special characters and other wierdness */
	public function escAttr($input)
	{
		// This was done so I could Unit test
		return esc_attr($input);
	}

	/**  Used in validate function */
	public function value_or_default($page, $attr, $userInput, $savedOptions)
	{
		if ($attr === "IncludedPostIds" || $attr === "ExcludedPostIds" || $attr === "HideRecsOnTheseIds" || $attr === "IncludeTheseIdsAsRecs") {
			if (isset($userInput[$page][$attr])) {
				// trim whitespace out of user input
				$temp = explode(",", $userInput[$page][$attr]);
				$userInput[$page][$attr] = implode(",", array_map("trim", $temp));
			}
		}
		return isset($userInput[$page][$attr]) ? $this->escAttr($userInput[$page][$attr]) : $savedOptions[$page][$attr];
	}

	// These are the options stored in the database
	public function getOptions()
	{
		return get_option($this->plugin_name);
	}

	// This was done so I could Unit test
	public function getPostTypes()
	{
		return get_post_types(array('public' => true));
	}

	public function getSelectedPostTypes($input, $tab)
	{
		$selected_post_types = array();
		$all_post_types = $this->getPostTypes();
		// This loops through
		foreach ($all_post_types as $key => $value) {
			$is_selected = (isset($input[$tab][$key]) && !empty($input[$tab][$key])) ? $key : null; // This won't work
			if (isset($is_selected)) {
				array_push($selected_post_types, $is_selected);
			}
		}
		return $selected_post_types;
	}

	public function validate($input)
	{
		// All the inputs on settings page - both in General and in Tray
		// They are both shared by the same form, so we have to make sure not to overwrite values that were not edited
		$valid = array();
		$recommend_these_types = $this->getSelectedPostTypes($input, 'general');
		$recs_on_these_page_types = $this->getSelectedPostTypes($input, 'tray');
		$options = $this->getOptions();

		// no longer true with addition of MyLibrary tab, but everything seems to work. afraid to touch it!
		$is_tray_tab = $input['Tab'] != 'General';

		// These checkboxes have special handling on 'General' Tab
		$valid['general']['RequireHTTPS'] = $is_tray_tab ? $options['general']['RequireHTTPS'] : (!empty($input['general']['RequireHTTPS'])) ? 1 : 0;
		$valid['general']['EnableTracking'] = $is_tray_tab ? $options['general']['EnableTracking'] : (!empty($input['general']['EnableTracking'])) ? 1 : 0;
		$valid['general']['PostTypesForRecs'] = $is_tray_tab && !empty($options['general']['PostTypesForRecs']) ? $options['general']['PostTypesForRecs'] : $recommend_these_types;

		$valid['tray']['RecsOnThesePageTypes'] = $is_tray_tab && !empty($options['tray']['RecsOnThesePageTypes']) ? $options['tray']['RecsOnThesePageTypes'] : $recs_on_these_page_types;
		$valid['tray']['EnableTray'] = $is_tray_tab ? $options['tray']['EnableTray'] : (!empty($input['tray']['EnableTray'])) ? 1 : 0;
		$valid['tray']['EnableTrayLoggedIn'] = $is_tray_tab ? $options['tray']['EnableTrayLoggedIn'] : (!empty($input['tray']['EnableTrayLoggedIn'])) ? 1 : 0;

		// General Tab Options
		$general_options = array('ClientAlias', 'ServiceKey', 'DataPrefix', 'CustomerIdCookie', 'ImageSize', 'DataMode');
		foreach ($general_options as $general_option) {
			$valid['general'][$general_option] = $this->value_or_default('general', $general_option, $input, $options);
		}

		// Tray Tab Options
		$tray_options = array('IncludedPostIds', 'ExcludedPostIds', 'HideRecsOnTheseIds', 'IncludeTheseIdsAsRecs', 'ButtonFontColor', 'ButtonBorderColor', 'TrayButtonColor', 'BubbleBGColor', 'PromptBackgroundColor', 'PromptFontColor', 'TrayTopBorderColor', 'TrayBGColor', 'TrayLayOut', 'TrayTitleText', 'ButtonText', 'IconBackgroundColor', 'IconSvgFillColor', 'IconType', 'TrayTitleColor', 'TrayFontSize', 'RecsBorderColor', 'RecsFontColor', 'RecsBackgroundColor', 'RecsFontSize', 'PromptOnDelay', 'PromptMessageNewUser', 'PromptMessageReturningUser', 'PromptMessageKnownUser', 'PromptOnScrollDepth', 'PromptLimitedTo', 'TrayOpenOnDelay', 'TrayAutoOpenLimitedTo', 'TrayOpenOnScrollDepth', 'PromotedRecId', 'PromotedRecTitle');
		foreach ($tray_options as $tray_option) {
			$valid['tray'][$tray_option] = $this->value_or_default('tray', $tray_option, $input, $options);
		}

		// My Library Tab Options
		$mylibrary_options = array('MyLibraryNumRecsMobile', 'MyLibraryNumRecsTablet', 'MyLibraryNumRecsDesktop', 'MyLibraryHeadingColor', 'MyLibraryHeadingFontSize', 'MyLibraryRecBorderColor', 'MyLibraryRecBackgroundColor', 'MyLibraryRecFontColor', 'MyLibraryRecFontSize');
		foreach ($mylibrary_options as $mylibrary_option) {
			$valid['mylibrary'][$mylibrary_option] = $this->value_or_default('mylibrary', $mylibrary_option, $input, $options);
		}

		return $valid;
	}

	public function add_image()
	{
		add_theme_support('post-thumbnails');
		add_image_size('custom_thumbnail', 320, 160, true);
	}
}
