<?php
/**
 * This is the 4Tell API to get all the data about posts and pages about specific clients
 * 
 * You can expand this to get User data as well, really anything in the WordPress database
 */


class Personalization_4tell_Export
{
	/**
	 * The ID of this plugin.
	 *
	 * @since    1.1.2
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.1.2
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Utils for this plugin.
	 *
	 * @since    1.1.3
	 * @access   private
	 * @var      string    $utils    T
	 */
	private $utils;

	/**
	 * Current options selected by user.
	 *
	 * @since    1.1.2
	 * @access   private
	 * @var      string    $options    Current options selected by the user.
	 */
	private $options;

	/**
	 * Ranges if they exist in the catalog call.
	 *
	 * @since    1.1.2
	 * @access   private
	 * @var      string    $ranges    Current ranges if in export.
	 */
	private $ranges;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.1.2
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct($plugin_name, $version, $utils)
	{
		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->utils = $utils;
		$this->options = get_option($this->plugin_name);

		// This connect this plugin to mysql database
		$this->connectToDatabase();
	}

	private function connectToDatabase()
	{
		$connection = mysqli_connect(DB_HOST, DB_USER, DB_PASSWORD);
		mysqli_select_db($connection, DB_NAME);
	}

	/** RangeType is "row" or "date" */
	private function setRange($ranges, $rangeType)
	{
		$this->ranges[$rangeType]["start"] = $rangeType === "row" ? (int)$ranges[0] - 1 : $ranges[0];
		$this->ranges[$rangeType]["end"] = $rangeType === "row" ? (int)$ranges[1] : $ranges[1];
	}

	private function validate_extra_fields($fields)
	{
		$fields_array = explode(",", $fields);
		foreach ($fields_array as $field) {
			if (!is_string($field)) {
				// pretty sure this never gets called
				$this->utils->senderror("Extra Fields must be strings", 401);
			}
		}
	}

	private function validate_Ranges($rowOrDateRange, $rangeType)
	{
		// range is either rowRange=1,5000 or dateRange=10-31-2018,11-31-2018
		$Range = explode(",", $rowOrDateRange);

		if (count($Range) && count($Range) != 2) {
			$this->utils->senderror("" . $rangeType . " range must be two comma separated values.", 401);
		} else if (count($Range) == 2) {
			if ($rangeType === "row") {
				if (!is_numeric($Range[0]) || !is_numeric($Range[1])) {
					$this->utils->senderror("RowRange must be two comma separated numbers. Ensure there are no quotes included.", 401);
				}
				if ((int)$Range[0] < 1 || (int)$Range[1] < 1) {
					$this->utils->senderror("RowRange values need to be greater than 1.", 401);
				}
			}
			if ($rangeType == "date") {
				if (!$this->utils->isaValidDate($Range[0]) || !$this->utils->isaValidDate($Range[1])) {
					$this->utils->senderror("DateRange must be in YYYY-MM-DD format.", 401);
				}
			}
			// sweet its in our format, lets store it in the class
			$this->setRange($Range, $rangeType); // $rangeType either dateRange or numOfDatabase rows
		}
	}

	private function check_call_isvalid($options, $params)
	{
		// Params from Url
		$clientAlias = $params['ClientAlias'];
		$serviceKey = $params['ServiceKey'];
		$dataGroup = $params['DataGroup'];
		$rowRange = $params['RowRange'];
		$dateRange = $params['DateRange'];
		$extraFields = $params['ExtraFields'];
		$resultType = esc_html($params['ResultType']);

		// Params from Plugin
		$adminClientAlias = $options['general']['ClientAlias'];
		$adminServiceKey = $options['general']['ServiceKey'];

		if ($clientAlias !== $adminClientAlias) {
			$this->utils->senderror("Client Alias does not match", 401);
		}

		if ($serviceKey !== $adminServiceKey) {
			$this->utils->senderror("Service key does not match", 401);
		}

		if ($dataGroup !== "Catalog") {
			$this->utils->senderror("Data group is invalid", 400);
		}

		if (!empty($dateRange)) {
			$this->validate_Ranges($dateRange, "date");
		}

		if (!empty($rowRange)) {
			$this->validate_Ranges($rowRange, "row");
		}
		$acceptedTypes = ["Data", "Count", "Query"];
		if (!empty($resultType) && !in_array($resultType, $acceptedTypes)) {
			$message = "ResultType: " . $resultType . " is not one of the allowed types. Try " . implode(", ", $acceptedTypes) . "";
			$this->utils->senderror($message, 401);
		}
		if (!empty($extraFields)) {
			$this->validate_extra_fields($extraFields);
		}
		if (count($this->all_selected_postTypes()) == 0) {
			$this->utils->senderror("No post types are selected. Please go to the admin panel to update what types are shown as recommendations.", 400);
		}
	}

	/** This checks to see if the api request is valid. It kills the function if it gets a failure */
	public function grab_catalog_data($params)
	{
		$this->check_call_isvalid($this->options, $params);
		return $this->getPostTypesAndData($params);
	}

	private function getPostTypesAndData($params)
	{
		$resultType = $params['ResultType'];
		$PostTypes = $this->all_selected_postTypes();
		$posts_as_strings = $this->utils->createCommaString($PostTypes); // formatted for SQL
		$qry = $this->createQuery($posts_as_strings, $params);

		if ($resultType === 'Query') {
			echo "{\"Query\": $qry }";
		} else {
			$posts = $this->queryDatabase($posts_as_strings, $params);
			if ($resultType === 'Count') {
				$postCount = count($posts);
				echo "{\"Count\": $postCount }";
			} else {
				$this->sendResponse($this->formatPosts($posts));
			}
		}
	}

	private function createQuery($postTypes, $requestParams)
	{
		$extraFields = $requestParams["ExtraFields"];
		// This global must be put inside the function
		global $wpdb;

		$ranges = $this->ranges;
		$rowOffset = $ranges['row']['start'];
		$rowCount = $ranges['row']['end'];
		$dateStart = $ranges['date']['start'];
		$dateEnd = $ranges['date']['end'];


		// Top Sellers will show as Recs if we dont include it in the feed
		$Included_Ids = explode(',', $this->options['tray']['IncludedPostIds']);
		$IncludeTheseIdsAsRecs = explode(',', $this->options['tray']['IncludeTheseIdsAsRecs']);
		$promoted_postID = array($this->options['tray']['PromotedRecId']);
		$all_included_ids = array_unique(array_merge($Included_Ids, $IncludeTheseIdsAsRecs, $promoted_postID));
		$includedIds = $this->utils->createCommaString($all_included_ids);

		// for each extra field, we need to have 
		// 1. a table alias (extraFieldColumns)
		// 2. a column in the SELECT clause
		// 3. a LEFT JOIN that treats the EAV field
		//    like a column (extraFieldLeftJoins)

		// ,extrafield0.meta_value AS "$extraFieldName"
		// ,extrafield1.meta_value AS "$extraFieldName"
		// LEFT JOIN (SELECT post_id, meta_value FROM wp_postmeta WHERE meta_key='{extraFieldName}') extrafield0 ON extrafield0.post_id=p.ID
		// LEFT JOIN (SELECT post_id, meta_value FROM wp_postmeta WHERE meta_key='{extraFieldName}') extrafield1 ON extrafield1.post_id=p.ID

		$extraFieldColumns = "";
		$extraFieldLeftJoins = "";

		if ($extraFields) {
			$fields = explode(",", $extraFields);
			if ($fields && count($fields) > 0) {
				for ($i = 0; $i < count($fields); $i++) {
					$field = trim($fields[$i]);
					$extraFieldColumns .= ",extrafield{$i}.meta_value AS '${field}' ";
					$extraFieldLeftJoins .= " 
					LEFT JOIN (
						SELECT * 
						FROM wp_postmeta 
						WHERE meta_key='{$field}') extrafield{$i} ON extrafield{$i}.post_id=p.ID
							";
				}
			}
		}


		$qs = "
			SELECT
				p.ID,
				p.post_title as Title,
				p.post_date as ActivatedDate,
				p.post_modified as ModifiedDate,
				p.post_type as PostType,
				p.post_excerpt as Description,
				p.post_parent as Parent,
				case
					when p.post_status = 'publish' then 1
					else 0
				end as Active,
				'' as URL,
				'' as ImageLink

				,(SELECT GROUP_CONCAT({$wpdb->terms}.name SEPARATOR ', ')
				    FROM 		{$wpdb->terms}
				    INNER JOIN 	{$wpdb->term_taxonomy} 			ON {$wpdb->terms}.term_id = {$wpdb->term_taxonomy}.term_id
				    INNER JOIN 	{$wpdb->term_relationships} wpr 	ON wpr.term_taxonomy_id = {$wpdb->term_taxonomy}.term_taxonomy_id
				    WHERE 		taxonomy = 'category'
				    AND 		p.ID = wpr.object_id
				) AS CategoryIDs

				,(SELECT GROUP_CONCAT({$wpdb->terms}.name SEPARATOR ', ')
				    FROM 		{$wpdb->terms}
				    INNER JOIN 	{$wpdb->term_taxonomy} 			ON {$wpdb->terms}.term_id = {$wpdb->term_taxonomy}.term_id
				    INNER JOIN 	{$wpdb->term_relationships} wpr 	ON wpr.term_taxonomy_id = {$wpdb->term_taxonomy}.term_taxonomy_id
				    WHERE 		taxonomy = 'post_tag'
				    AND 		p.ID = wpr.object_id
				) AS PostTag
				, '' as Excluded
				{$extraFieldColumns}

			FROM
				{$wpdb->posts} p
			{$extraFieldLeftJoins}
			WHERE p.post_type IN ({$postTypes}) OR p.ID IN (${includedIds}) AND p.post_password = ''
		";


		$sqlParams = array();

		// Added where statements for date range if both dates are specified AND p.post_type = '{$postType}'
		if ($dateStart && $dateEnd) {
			$qs .=
				"
				AND p.post_modified >= %s
				AND p.post_modified <= %s
			";
			array_push($sqlParams, $dateStart);
			array_push($sqlParams, $dateEnd);
		}

		// Add Order By statement
		$qs .=
			"	ORDER BY
				p.post_date_gmt DESC
		";

		// Add Limit statement if start and count are both specified
		if ($rowOffset >= 0 && $rowCount) {
			$qs .= " LIMIT %d,%d";
			array_push($sqlParams, $rowOffset);
			array_push($sqlParams, $rowCount);
		}
		return $wpdb->prepare($qs, $sqlParams); // Prepare the statement based on passed in parameters
	}

	private function queryDatabase($postTypes, $queryParams)
	{
		global $wpdb;
		return $wpdb->get_results($this->createQuery($postTypes, $queryParams));
	}

	private function all_selected_postTypes()
	{
		$PostTypesForRecs = $this->options['general']['PostTypesForRecs']; // "where are recs displayed"
		$RecsOnThesePageTypes = $this->options['tray']['RecsOnThesePageTypes']; // "what types are displayed as recs"
		return array_unique(array_merge($PostTypesForRecs, $RecsOnThesePageTypes));
	}

	private function isExcluded($post)
	{
		$excluded_postIds = $this->options['tray']['ExcludedPostIds']; // these post ID's were explicitly excluded by the plugin Admin
		$includedPostTypesForRecs = $this->options['general']['PostTypesForRecs']; // "where are recs displayed"

		$excluded_by_id =  in_array($post->ID, explode(",", $excluded_postIds));
		$excluded_by_type = !in_array($post->PostType, $includedPostTypesForRecs);
		return $excluded_by_id || $excluded_by_type;
	}

	/** Take the sql response, builds the header JSON, and formats each post as JSON as well. */
	private function formatPosts($posts)
	{
		$data_prefix = $this->options['general']['DataPrefix'];

		foreach ($posts as $post) {
			$pu = parse_url(get_permalink($post->ID));
			$post->URL = $pu['path'];
			$post->Excluded = $this->isExcluded($post) ? "true" : "false";
			$post->ImageLink = $this->getImageLink($post->ID, true);
			$parentId = $post->Parent;
			// give prefix to ParentId if there is one, '' if null or 0
			$post->Parent = empty($parentId) || $parentId === 0 ? '' : $data_prefix . "" . $parentId;
			$post->ID = $data_prefix . "" . $post->ID;
		}
		return $posts;
	}

	private function sendResponse($posts)
	{
		if (!$posts || count($posts) == 0) {
			// surely a better way to handle no results, but right now it throws an error so im just fixing that. temporary stopgap
			echo "No posts found";
			return;
		}

		// first object in response is the field names
		$fields = array_keys(get_object_vars($posts[0])); // Get the field column names
		$last_field = end($fields);
		echo '[';
		echo 	'["';
		echo  		implode('","', $fields);
		echo 	'"]';

		// then loop through and return the values for those fields
		foreach ($posts as $post) {
			echo ",[";
			foreach ($post as $key => $value) {
				echo json_encode($value != null ? $value : ''); // Boost/Generator needs empty strings for nulls
				if ($key !== $last_field) {
					echo ', '; // we're not at the end yet!
				}
			}
			echo "]";
		}
		echo ']';
	}

	public function grab_params_data($request)
	{
		$params = $request->get_params();
		$catalogData = $this->grab_catalog_data($params);
		return $catalogData;
	}

	public function createArg($description, $type, $required)
	{
		return array(
			'required' => $required,
			'description' => esc_html($description),
			'type' => $type
		);
	}

	public function catalog_4tell_args()
	{
		$args = array();
		$args['ClientAlias'] = $this->createArg("Clients unquie identifier provided by 4-Tell.", "string", true);
		$args['ServiceKey'] = $this->createArg("Clients unquie key provided by 4-Tell.", "string", true);
		$args['DataGroup'] = $this->createArg("Data group for extraction.", "string", false);
		$args['ResultType'] = $this->createArg("Data group for extraction.", "string", false);
		$args['DataRange'] = $this->createArg("Date range to grab data from. Needs to be comma delimited and only two values that are both numbers.", "string", false);
		$args['RowRange'] = $this->createArg("Row range to grab data from. Needs to be comma delimited and only two values that are both numbers.", "string", false);
		$args['ExtraFields'] = $this->createArg("A comma delimited list of extra fields", "string", false);
		return $args;
	}

	public function register_endpoints()
	{
		$this->register_getCatalog_endpoint();
		$this->register_options_endpoint();
		$this->register_getRecentlyAdded_endpoint();
		$this->register_getVersion_endpoint();
	}

	// {{domain}}/wp-json/personalization/v1/getCatalog
	private function register_getCatalog_endpoint()
	{
		register_rest_route("personalization/v1", "/getCatalog", array(
			'method' => 'GET',
			'callback' => array($this, "grab_params_data"),
			'args' => $this->catalog_4tell_args()
		));
	}

	// {{domain}}/wp-json/personalization/v1/recentPosts
	private function register_getRecentlyAdded_endpoint()
	{
		register_rest_route("personalization/v1", "/recentPosts", array(
			'method' => 'GET',
			'callback' => array($this, "getRecentlyAddedPosts"),
		));
	}

	// {{domain}}/wp-json/personalization/v1/getVersion
	private function register_getVersion_endpoint()
	{
		register_rest_route("personalization/v1", "/version", array(
			'method' => 'GET',
			'callback' => array($this, "getVersion")
		));
	}

	// {{domain}}/wp-json/personalization/v1/admin/settings
	private function register_options_endpoint()
	{
		register_rest_route("personalization/v1", "/admin/settings", array(
			'method' => 'GET',
			'callback' => array($this, "adminSettings"),
			'args' => array(),
		));
	}

	public function getVersion()
	{
		global $wp_version;
		global $required_php_version;
		global $required_mysql_version;
		$php_version = phpversion();
		$plugin_version = $this->version;
		return array(
			'WordPress' => $wp_version,
			'PHP' => $php_version,
			'Plugin' => $plugin_version,
			'Required PHP Version' => $required_php_version,
			'Required MySQL Version' => $required_mysql_version,
		);
	}

	public function adminSettings()
	{
		$options = $this->options;
		unset($options['general']['ServiceKey']);
		return $options;
	}

	public function getRecentlyAddedPosts()
	{
		$args = array(
			'numberposts' => 30,
			'orderby' => 'post_date',
			'order' => 'DESC',
			'post_type' => 'post',
			'post_status' => 'publish',
			'suppress_filters' => true
		);

		$recent_posts = wp_get_recent_posts($args, ARRAY_A);
		return $this->formatRecentPosts($recent_posts);
	}

	private function formatRecentPosts($posts)
	{
		foreach ($posts as &$post) {
			// reassign property names to match a response from Boost
			$post['id'] = $post['ID'];
			$post['url'] = $post['guid'];
			$post['imageUrl'] = $this->getImageLink($post["ID"], false);
			$post['name'] = $post['post_title'];
			$post['description'] = $post['post_excerpt'];
			$post['type'] = $post['post_type'];
			// remove unnecessary and renamed fields
			unset($post["post_content"]);
			unset($post["ID"]);
			unset($post["post_excerpt"]);
			unset($post["post_title"]);
			unset($post["post_parent"]);
			unset($post["comment_status"]);
			unset($post["post_type"]);
			unset($post["post_date"]);
			unset($post["filter"]);
			unset($post["post_date_gmt"]);
			unset($post["post_name"]);
			unset($post["post_status"]);
			unset($post["post_password"]);
			unset($post["pinged"]);
			unset($post["ping"]);
			unset($post["to_ping"]);
			unset($post["ping_status"]);
			unset($post["guid"]);
			unset($post["post_mime_type"]);
			unset($post["comment_count"]);
			unset($post["menu_order"]);
			unset($post["post_modified"]);
			unset($post["post_modified_gmt"]);
			unset($post["post_content_filtered"]);
			unset($post["post_author"]);
		}
		return $posts;
	}

	private function getImageLink($postId, $scrubHttp)
	{
		$imageSize = $this->options['general']['ImageSize'];
		$link = wp_get_attachment_image_src(get_post_thumbnail_id($postId), $imageSize);
		if (isset($link) && is_array($link) && count($link)) {
			return $scrubHttp ? preg_replace('#^(http|https):#', '', $link[0]) : $link[0]; // remove http/https off the url
		}
		return "";
	}
}
