<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://get4tell.com
 * @since      1.0.0
 *
 * @package    Personalization_4tell
 * @subpackage Personalization_4tell/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Personalization_4tell
 * @subpackage Personalization_4tell/includes
 * @author     Chris Reeder <chris.reeder@4-tell.com>
 */
class Personalization_4tell
{

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Personalization_4tell_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct()
	{
		if (defined('PLUGIN_NAME_VERSION')) {
			$this->version = PLUGIN_NAME_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'Adaptive-Web';

		$this->load_dependencies();
		$this->set_locale();


		$this->define_api_hooks();
		$this->define_admin_hooks();
		$this->define_public_hooks();
		$this->create_options_table();
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Personalization_4tell_Loader. Orchestrates the hooks of the plugin.
	 * - Personalization_4tell_i18n. Defines internationalization functionality.
	 * - Personalization_4tell_Admin. Defines all hooks for the admin area.
	 * - Personalization_4tell_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies()
	{

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-personalization-4tell-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-personalization-4tell-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/class-personalization-4tell-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'public/class-personalization-4tell-public.php';



		/**
		 * The class responsible for defining all actions that occur in the export api
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/class-personalization-4tell-utils.php';

		/**
		 * The class responsible for defining all actions that occur in the export api
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/class-personalization-4tell-export.php';

		$this->loader = new Personalization_4tell_Loader();
	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Personalization_4tell_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale()
	{

		$plugin_i18n = new Personalization_4tell_i18n();

		$this->loader->add_action('plugins_loaded', $plugin_i18n, 'load_plugin_textdomain');
	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.1.2
	 * @access   private
	 */
	private function define_api_hooks()
	{
		$plugin_utils = new Personalization_4tell_Utils($this->get_plugin_name(), $this->get_version());
		// passing in utils to the class this way
		$plugin_api = new Personalization_4tell_Export($this->get_plugin_name(), $this->get_version(), $plugin_utils);

		// Create Api
		$this->loader->add_action('rest_api_init', $plugin_api, 'register_endpoints');
	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks()
	{

		$plugin_admin = new Personalization_4tell_Admin($this->get_plugin_name(), $this->get_version());

		$this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_styles');
		$this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts');

		// Admin Area
		$this->loader->add_action('admin_init', $plugin_admin, 'options_update');
		$this->loader->add_action('admin_menu', $plugin_admin, 'add_admin_menu');

		// Settings Links For The Plugin
		$plugin_basename = plugin_basename(plugin_dir_path(__DIR__)  . 'personalization-4tell.php');
		$this->loader->add_filter('plugin_action_links_' . $plugin_basename, $plugin_admin, 'add_action_links');

		// Create Image
		$this->loader->add_action('init', $plugin_admin, 'add_image');

		// Update Function
		$this->loader->add_action('init', $plugin_admin, 'load_update_checker');
		$this->loader->add_action('init', $plugin_admin, 'checkForUpdates');
	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks()
	{

		$plugin_public = new Personalization_4tell_Public($this->get_plugin_name(), $this->get_version());

		$this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_styles');
		$this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_scripts');

		// Loads Loader
		$this->loader->add_action('wp_head', $plugin_public, 'add_4Tell_loader');

		// Load data div
		$this->loader->add_action('wp_footer', $plugin_public, 'add_4Tell_footer');

		// Widget Area
		// $this->loader->add_action( 'widgets_init', $plugin_public, 'include_widget');
		// $this->loader->add_action( 'widgets_init', $plugin_public, 'category_widget');

	}

	/**
	 * Creates a placeholder for the options table when none exist. Solves DASH-3851
	 * Settings aren't lost or reset upon Deactivate/Reactivate.
	 */
	private function create_options_table()
	{
		if (!get_option($this->get_plugin_name())) {
			// this is the very first time someone installs our plugin
			$this->update_options_table($this->general_defaults(), $this->tray_defaults(), $this->mylibrary_defaults());
		} else {
			// here if we release a new feature to an exisiting customer, they already have an options table.
			// but, we want to inject a default value for our new feature, and not overwrite any previous values
			$this->maybe_add_default_values();
		}
	}


	private function maybe_add_default_values()
	{
		$options_table = get_option($this->get_plugin_name());

		$tray_options_to_add = array();
		foreach ($this->tray_defaults() as $name => $value) {
			// they dont have this value in their table yet, lets give them our default!
			if (!isset($options_table['tray'][$name])) {
				$tray_options_to_add[$name] = $value;
			}
		}
		$general_options_to_add = array();
		foreach ($this->general_defaults() as $name => $value) {
			// they dont have this value in their table yet, lets give them our default!
			if (!isset($options_table['general'][$name])) {
				$general_options_to_add[$name] = $value;
			}
		}

		$mylibrary_options_to_add = array();
		foreach ($this->mylibrary_defaults() as $name => $value) {
			// they dont have this value in their table yet, lets give them our default!
			if (!isset($options_table['mylibrary'][$name])) {
				$mylibrary_options_to_add[$name] = $value;
			}
		}


		if (count($tray_options_to_add) > 0 || count($general_options_to_add) > 0 || count($mylibrary_options_to_add) > 0) {
			/** This will wipe the table and insert all new values, so we must extract all data, merge new values, and the rewrite to the table */

			$saved_tray_options = $options_table['tray'];
			$saved_general_options = $options_table['general'];
			$saved_mylibrary_options = $options_table['mylibrary'];

			$all_tray_options = array_merge($saved_tray_options, $tray_options_to_add);
			$all_general_options = array_merge($saved_general_options, $general_options_to_add);
			$all_mylibrary_options = array_merge($saved_mylibrary_options, $mylibrary_options_to_add);
			$this->update_options_table($all_general_options, $all_tray_options, $all_mylibrary_options);
		}
	}

	private function update_options_table($generalOps, $trayOps, $mylibraryOps)
	{
		update_option($this->get_plugin_name(), array('general' => $generalOps, 'tray' => $trayOps, 'mylibrary' => $mylibraryOps));
	}

	private function general_defaults()
	{
		return array();
	}

	private function tray_defaults()
	{
		$blue = '#013D5B';
		$white = '#fff';
		$darkGrey = '#7B7B7B';
		$black = '#000000';
		$red = 'rgb(237, 63, 68)';
		return array(
			// prompt defaults
			'PromptLimitedTo' => 'AllPagesUntilDismissed',
			'PromptMessageNewUser' => "Welcome! I predict you will enjoy these three items",
			'PromptMessageReturningUser' => "Welcome back! I predict you will like these three items",
			'PromptMessageKnownUser' => "Welcome {name}! I predict you will like these three items",
			'PromptOnDelay' => '4',
			'PromptOnScrollDepth' => 'HalfWayDown',
			'PromptFontColor' => $white,
			'PromptBackgroundColor' => $red,
			// tray auto open defaults
			'TrayAutoOpenLimitedTo' => 'AllPagesUntilDismissed',
			'TrayOpenOnDelay' => '20',
			'TrayLayOut' => 'Horizontal',
			'TrayOpenOnScrollDepth' => 'Bottom',
			// style defaults
			'ButtonText' => 'Check out what we picked for you',
			'TrayTitleText' => 'Also viewed with this post',
			'PromotedRecTitle' => 'Check out this upcoming event',
			'ButtonFontColor' => $white,
			'ButtonBorderColor' => $white,
			'IconSvgFillColor' => $white,
			'TrayBGColor' => $white,
			'TrayTopBorderColor' => $blue,
			'TrayButtonColor' => $blue,
			'IconBackgroundColor' => $blue,
			'TrayTitleColor' => $blue,
			'TrayFontSize' => '18px',
			'RecsFontSize' => '14px',
			'BubbleBGColor' => $red,
			'RecsFontColor' => $black,
			'RecsBackgroundColor' => $white,
			'RecsBorderColor' => $darkGrey,
		);
	}

	private function mylibrary_defaults()
	{
		$white = '#fff';
		$darkGrey = '#7B7B7B';
		$black = '#000000';

		return array(
			'MyLibraryNumRecsMobile' => 2,
			'MyLibraryNumRecsTablet' => 4,
			'MyLibraryNumRecsDesktop' => 4,
			'MyLibraryHeadingColor' => $black,
			'MyLibraryHeadingFontSize' => "16px",
			'MyLibraryRecBorderColor' => $darkGrey,
			'MyLibraryRecBackgroundColor' => $white,
			'MyLibraryRecFontColor' => $black,
			'MyLibraryRecFontSize' => '14px'
		);
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run()
	{
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name()
	{
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Personalization_4tell_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader()
	{
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version()
	{
		return $this->version;
	}
}
