<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://get4tell.com
 * @since      1.0.0
 *
 * @package    Personalization_4tell
 * @subpackage Personalization_4tell/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Personalization_4tell
 * @subpackage Personalization_4tell/public
 * @author     Chris Reeder <chris.reeder@4-tell.com>
 */
class Personalization_4tell_Public
{

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $options;


	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct($plugin_name, $version)
	{

		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->options = get_option($this->plugin_name);
	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles()
	{

		wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) . 'css/personalization-4tell-public.css', array(), $this->version, 'all');
	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts()
	{

		wp_enqueue_script($this->plugin_name, plugin_dir_url(__FILE__) . 'js/personalization-4tell-public.js', array('jquery'), $this->version, false);
	}

	/**
	 * This function inserts our loader inside the head tag. We needed to change this compared to how our other loader works due to
	 * a lot of wordpress sites combine scripts. I put the two variables needed by the script on the window, which is not preferable
	 * but is the easiest when using. I put the variables under NewLoader just in case we are running old recs on a site so there isn't
	 * any collisions with the old global object.
	 */
	public function add_4Tell_loader()
	{
		if ($this->options['general']['EnableTracking']) {
			echo "<script type='text/javascript'>(function(_4TellBoost){if(window){window._4TellBoost.NewLoader = {}; window._4TellBoost.NewLoader.ClientAlias ='" . $this->options['general']['ClientAlias'] . "'; window._4TellBoost.NewLoader.DataMode ='" . $this->options['general']['DataMode'] . "';}})((window._4TellBoost = window._4TellBoost || {}))</script>";
			// use the below line to making changes to the loader and testing locally
			// echo "<script type='text/javascript' async src='" . plugin_dir_url(__FILE__) . "/js/loader.js'></script>";
			echo "<script type='text/javascript' async src='https://4tellcdn.azureedge.net/sites/actonact/wordpress-plugin/resources/loader.js'></script>";
		}
	}

	/** Did the admin enable the tray on this specific page type? */
	private function should_enable_tray()
	{
		$currentPostType = get_post_type();
		$loggedIn = is_user_logged_in();

		$tray_options = $this->options['tray'];
		$recs_on_these_pages = is_array($tray_options['RecsOnThesePageTypes']) ? $tray_options['RecsOnThesePageTypes'] : array();
		$IncludedPostIds = explode(',', $tray_options['IncludedPostIds']);
		$HideRecsOnTheseIds = explode(',', $tray_options['HideRecsOnTheseIds']);
		$PageId = get_the_ID();

		$disabled_by_postId = in_array($PageId, $HideRecsOnTheseIds); // admin said to not display recs on these specific pages
		$enabled_by_postType = in_array($currentPostType, $recs_on_these_pages); // admin said to display recs on all these post types
		$enabled_by_postId = in_array($PageId, $IncludedPostIds); // specifically enabled by an Admin to show recs on this exact page

		if ($enabled_by_postType || $enabled_by_postId) {
			if ($disabled_by_postId) {
				return false;
			} else if ($tray_options['EnableTrayLoggedIn']) {
				return $loggedIn;
			} else if ($tray_options['EnableTray']) {
				return true;
			}
		}
		return false;
	}

	/**
	 * This is what will add divs depending on what is activated inside the plugin
	 */
	public function add_4Tell_footer()
	{
		if ($this->should_enable_tray()) {
			// maybe add the tray if on the correct page
			echo "<div id='tray_4Tell'></div>";
		}
		echo $this->create_main_div();
	}

	private function create_main_div()
	{
		$options = $this->options;
		$options['general']['PageId'] =  $this->get_page_ID();
		/**
		 * Here I'm assuming there is going to be future logic of when you want to track something as content
		 * or product or something else. This will work for now but might need it's own function to figure out
		 * when something is content or not.
		 */
		$options['general']['IsContent'] = true;

		$customer_id = $this->get_customer_id();
		$customer_id_attribute = $customer_id ? "data-customer='$customer_id'" : null;

		$cleaned_options = $this->clean_options($options);
		return "<div id='data_4Tell' " . $customer_id_attribute . "  data-info='" . json_encode($cleaned_options) . "'></div>";
	}

	private function get_page_ID()
	{
		// looking for page/post ID this way due to DASH-3877
		$classes = get_body_class();
		foreach ($classes as $class) {
			if (strpos($class, "page-id-") === 0 || strpos($class, "postid-") === 0) {
				return get_the_ID();
			}
		}
		return "";
	}

	/** Grabs customer id from the cookie based off the field the admin entered in the "General" Settings tab */
	private function get_customer_id()
	{
		$cookie_name = $this->options['general']['CustomerIdCookie'];
		return $_COOKIE[$cookie_name];
	}

	/** The JS bundle doesn't need any of this information, so lets clean it up a bit */
	private function clean_options($options)
	{
		$tray_options = &$options['tray'];
		$general_options = &$options['general'];

		unset($general_options['ServiceKey']); // definitely hide you
		unset($general_options['ImageSize']);
		unset($general_options['PostTypesForRecs']);
		unset($general_options['RequireHTTPS']);
		unset($general_options['CustomerIdCookie']);


		unset($tray_options['RecsOnThesePageTypes']);
		unset($tray_options['EnableTray']);
		unset($tray_options['EnableTrayLoggedIn']);
		unset($tray_options['IncludedPostIds']);
		unset($tray_options['ExcludedPostIds']);
		unset($tray_options['HideRecsOnTheseIds']);
		unset($tray_options['IncludeTheseIdsAsRecs']);

		if ($tray_options['PromptLimitedTo'] == 'Disabled') {
			unset($tray_options['PromptOnDelay']);
			unset($tray_options['PromptOnScrollDepth']);
			unset($tray_options['PromptMessageNewUser']);
			unset($tray_options['PromptMessageReturningUser']);
			unset($tray_options['PromptMessageKnownUser']);
			unset($tray_options['PromptBackgroundColor']);
			unset($tray_options['PromptFontColor']);
		}
		if ($tray_options['TrayAutoOpenLimitedTo'] == 'Disabled') {
			unset($tray_options['TrayOpenOnDelay']);
			unset($tray_options['TrayOpenOnScrollDepth']);
		}
		if (empty($tray_options['PromotedRecId'])) {
			unset($tray_options['PromotedRecTitle']);
		} else {
			if ($general_options['DataPrefix']) {
				// JS bundle needs the PromotedPostId with the  prefix
				$tray_options['PromotedRecId'] = $general_options['DataPrefix'] . "" . $tray_options['PromotedRecId'];
			}
		}
		if (empty($tray_options['IconType'])) {
			// using the default button, hide the SVG styles
			unset($tray_options['IconBackgroundColor']);
			unset($tray_options['IconSvgFillColor']);
		} else {
			// we're using an SVG here, so hide all Button styles
			unset($tray_options['ButtonText']);
			unset($tray_options['TrayButtonColor']);
			unset($tray_options['ButtonFontColor']);
			unset($tray_options['ButtonBorderColor']);
		}
		return $options;
	}

	// I'm going to be bringing this back.
	// public function include_widget(){
	// 	include_once('partials/class-personalization-4tell-widget.php');
	// }

	// public function category_widget(){
	// 	register_widget('Recommendation_Tray_Widget');
	// }

}
